"""Gron's command line interface (CLI).

Attributes
----------
parser : argparse.ArgumentParser

"""

# remove when we don't support py38 anymore
from __future__ import annotations

import argparse
import sys

from gron import __VERSION__, gron

parser = argparse.ArgumentParser()
parser.add_argument(
    'file',
    help='A JSON file, if not given gron reads from STDIN',
    nargs='?',
    type=argparse.FileType('r'),
    default=sys.stdin,
)


parser.add_argument("--version", action="version", version=__VERSION__)


def main() -> None:
    """Gron's CLI.

    This method reads the arguments for the command line interface and
    runs `gron`.

    """
    args = parser.parse_args()
    data = args.file.read()
    print(gron(data))


if __name__ == '__main__':
    main()



"""Gron's core functions."""

# remove when we don't support py38 anymore
from __future__ import annotations

import json
from typing import Any


def walk(node: Any, name: str) -> str:
    """Translate Python element to JSON.

    This method recursively visits each element of a Python object and
    returns the JSON representation.

    Parameters
    ----------
    node
        A python object (e.g. dict, list, int, etc)
    name
        The name (i.e. path) of the parent element

    Returns
    -------
    str
        Transformed JSON for this element

    """
    if node is None:
        return f"{name} = null;"
    elif isinstance(node, bool):
        return f"{name} = {str(node).lower()};"
    elif isinstance(node, str):
        return f'{name} = "{node}";'
    elif isinstance(node, bytes):
        return f'{name} = "{node!r}";'
    elif isinstance(node, dict):
        res = []
        res.append(f"{name} = {{}};")
        for k, v in sorted(node.items()):
            res.append(walk(v, name + convert('.' + k)))
        return '\n'.join(sorted(res))
    elif isinstance(node, list | tuple):
        res = []
        res.append(f"{name} = [];")
        for i, e in enumerate(node):
            res.append(walk(e, name + convert(str([i]))))
        return '\n'.join(res)

    else:
        return f"{name} = {node!r};"


def convert(name: str) -> str:
    """Convert path name into valid JSON.

    Parameters
    ----------
    name
        a path name

    Returns
    -------
    str
        valid JSON path

    """
    if '-' in name or ' ' in name:
        return f'["{name[1:]}"]'
    return name


def gron(input_: str) -> str:
    """Transform JSON into parseable str.

    This method takes a JSON string and transforms it into a grepable
    equivalent form.

    Parameters
    ----------
    input_
        JSON

    Returns
    -------
    str
        Transformed output

    """
    python = json.loads(input_)
    output = walk(python, 'json')
    return output



"""Test gron."""

from gron import gron


def test_github_data() -> None:
    """Test github data."""
    with open('./tests/data/github.json') as fh:
        IN = fh.read().strip()
    with open('./tests/data/github.gron') as fh:
        OUT = fh.read().strip()
    out = gron(IN)
    assert out == OUT


# not implemented yet
# def test_stream():
#     with open('./tests/data/stream.json', 'r') as fh:
#         IN = fh.read().strip()
#     with open('./tests/data/stream.gron', 'r') as fh:
#         OUT = fh.read().strip()
#     out = gron(IN)
#     assert out == OUT


def test_one() -> None:
    """Test one."""
    with open('./tests/data/one.json') as fh:
        IN = fh.read().strip()
    with open('./tests/data/one.gron') as fh:
        OUT = fh.read().strip()
    out = gron(IN)
    assert out == OUT


def test_two() -> None:
    """Test two."""
    with open('./tests/data/two.json') as fh:
        IN = fh.read().strip()
    with open('./tests/data/two.gron') as fh:
        OUT = fh.read().strip()
    out = gron(IN)
    assert out == OUT


def test_three() -> None:
    """Test three."""
    with open('./tests/data/three.json') as fh:
        IN = fh.read().strip()
    with open('./tests/data/three.gron') as fh:
        OUT = fh.read().strip()
    out = gron(IN)
    assert out == OUT


def test_ugly() -> None:
    """Test ugly."""
    with open('./tests/data/ugly.json') as fh:
        IN = fh.read().strip()
    with open('./tests/data/ugly.gron') as fh:
        OUT = fh.read().strip()
    out = gron(IN)
    assert out == OUT
